
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <limits>
#include <cmath>

//#include <boost/serialization/nvp.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>

using namespace std;

class data
{
  double __v1, __v2;
  
public:
  bool values_are_ok () const
  {
    return isnormal (__v1) &&  isnormal (__v2);
  }

  data ()
  {
    __v1 = drand48 () * 1.e-10;
    __v2 = drand48 () * 1.e+6;
  }

  data (double v1_, double v2_)
  {
    __v1 = v1_;
    __v2 = v2_;
  }

  void inf ()
  {
    __v1 = -std::numeric_limits<double>::infinity ();
    __v2 = std::numeric_limits<double>::infinity ();
  }

  void nan ()
  {
    __v1 = std::numeric_limits<double>::quiet_NaN ();
    __v2 = std::numeric_limits<double>::quiet_NaN ();
  }

  double get_v1 () const
  {
    return __v1;
  }

  double get_v2 () const
  {
    return __v2;
  }

private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
  {
    ar & boost::serialization::make_nvp ("v1", __v1);
    ar & boost::serialization::make_nvp ("v2", __v2);
  }
};

ostream & operator<<(ostream & out_, const data & d_)
{
  out_ << d_.get_v1 () << ' ' << d_.get_v2 ();
  return out_;
}

int 
main (void)
{
  long seed = 314159;
  srand48 (seed);

  {
    data d1, d2, d3;   
    d1.inf ();
    d2.nan ();
    clog << "d1=" << d1 << endl;
    clog << "d2=" << d2 << endl;
    clog << "d3=" << d3 << endl;
    
    {
      clog << "Store in text archive..." << endl;
      std::ofstream ofs ("double.txt");
      boost::archive::text_oarchive oa (ofs);
      oa & d1;
      oa & d2;
      oa & d3;
    }
    {
      clog << "Store in XML archive..." << endl;
      std::ofstream xofs ("double.xml");
      boost::archive::xml_oarchive xoa (xofs);
      xoa &  boost::serialization::make_nvp ("d1", d1);
      xoa &  boost::serialization::make_nvp ("d2", d2);
      xoa &  boost::serialization::make_nvp ("d3", d3);
    }
  }
  
  {
    data d1, d2, d3;
    clog << "Load from text archive..." << endl;
    std::ifstream ifs ("double.txt");
    boost::archive::text_iarchive ia (ifs);
    ia & d1;
    ia & d2;
    ia & d3;
    clog << "d1=" << d1 << endl;
    clog << "d2=" << d2 << endl;
    clog << "d3=" << d3 << endl;
  }

  {
    data d1, d2, d3;
    clog << "Load from XML archive..." << endl;
    std::ifstream ifs ("double.xml");
    boost::archive::xml_iarchive ia (ifs);
    ia & boost::serialization::make_nvp ("d1", d1);
    ia & boost::serialization::make_nvp ("d2", d2);
    ia & boost::serialization::make_nvp ("d3", d3);
    clog << "d1=" << d1 << endl;
    clog << "d2=" << d2 << endl;
    clog << "d3=" << d3 << endl;
  }

  return 0;
};
